#!/bin/bash
#
####################################################################
# Script        : rosetta.sh (Bash) 	                           #
# Release       : 1.1                                              #
# Last Update   : July 07, 2006                                    #
# ================================================================ #
# Author        : Giuseppe La Rocca (giuseppe.larocca@ct.infn.it)  #
# Organization  : Physics Institute for Nuclear Research           #
#               : INFN - University of Catania                     #
# Address       : Via S. Sofia, 64                                 #
#               : 95123 Catania (CT) - ITALY                       #
# Phone         : (+39) 095.378.54.73                              #
####################################################################
#
export LCG_CATALOG_TYPE=lfc
export LFC_HOST=lfc-gilda.ct.infn.it
#
# DATA_INPUT variable contains the folder name 
# of data that will be used during the 
# ROSETTA's computation.
#
DATA_INPUT=$1
PARAMETER=$3
LOCAL_DIR=`pwd`
#
#######################################
#     Download all the needed files   #
#     to run ROSETTA's application    #
#     from the LFC Catalog.           #
#######################################
if [ ${LCG_CATALOG_TYPE} = "lfc" ] ; then
	echo "Using LFC Catalog type."
	echo;echo "Download rosetta's database.."
	echo lcg-cp -v --vo gilda lfn:/grid/gilda/ROSETTA/rosetta_database.tar.gz file:`pwd`/rosetta_database.tar.gz
	lcg-cp -v --vo gilda lfn:/grid/gilda/ROSETTA/rosetta_database.tar.gz file:`pwd`/rosetta_database.tar.gz
	echo;echo "Download the source.."
	echo lcg-cp -v --vo gilda lfn:/grid/gilda/ROSETTA/rosetta_source.tar.gz file:`pwd`/rosetta_source.tar.gz
	lcg-cp -v --vo gilda lfn:/grid/gilda/ROSETTA/rosetta_source.tar.gz file:`pwd`/rosetta_source.tar.gz
fi
#
#########################################
#       Untar the DATABASE in the       #
#       local path of the worker        #
#       node.                           #
#########################################
echo;echo "+-------------------------------------------+ Untar the DATABASE."
tar zxvf `pwd`/rosetta_database.tar.gz
echo "+-------------------------------------------+ Untar the DATABASE."
#
#########################################
#       Untar the SOURCE in the         #
#       local path of the worker        #
#       node.                           #
#########################################
echo;echo "+-------------------------------------------+ Untar the SOURCE."
tar zxvf `pwd`/rosetta_source.tar.gz
echo "+-------------------------------------------+ Untar the SOURCE."
#
#########################################
#       Untar the input-data in the     #
#       local path of the worker        #
#       node.                           #
#########################################
#
echo;echo "+-------------------------------------------+ Untar the INPUT-DATA."
echo tar zxvf `pwd`/${DATA_INPUT}
tar zxvf `pwd`/${DATA_INPUT}
#Extract the name of the new input-data..
NEW_FOLDER_DATA_INPUT=`echo ${DATA_INPUT} | awk -F'.tar.gz' '{print $1}'`
#..and rename it with the name 2ptl.
mv ${NEW_FOLDER_DATA_INPUT} `pwd`/2ptl >/dev/null 2>&1
echo "+-----------------------------------------------------------------+"

echo;echo "==================> Content of working directory <=================="
ls -al `pwd`
WORKING_DIR=`pwd`
echo
#
# Change the permission and run pFOLD.lnx script.
cd `pwd`/rosetta_source
chmod +x pFOLD.lnx
#
cd ${LOCAL_DIR}
ls -al ${LOCAL_DIR}
echo;echo "+--------------------------------------------------------+ START ROSETTA at " `date`
echo "#########################################################"
echo "#							#"
echo "# 	@@@@ @@@@@ @@@@ @@@@ @@@@@ @@@@@ @@@@@  	#"
echo "# 	@  @ @   @ @    @      @     @   @   @		#"
echo "# 	@@@@ @   @ @@@@ @@@    @     @   @@@@@		#"
echo "# 	@ @  @   @    @ @      @     @   @   @		#"
echo "# 	@  @ @@@@@ @@@@ @@@@   @     @   @   @		#"
echo "#							#"
echo "#########################################################"
echo
dir=`pwd`
compiler=$2

if  [ "X${compiler}" = "X" ] ; then
	compiler="lnx"
fi

# Make a test dir test
/bin/rm -rf `pwd`/test
mkdir test

# Link the executable if needed
/bin/rm -rf pFOLD.${compiler}
ln -s ${dir}/rosetta_source/pFOLD.${compiler} pFOLD.${compiler}

# Enter the test directory
cd `pwd`/test
`pwd`
echo ab-initio test...
echo ../pFOLD.${compiler} aa 2ptl _ -constant_seed -nstruct 1 -status -paths ../2ptl/paths.txt > 2ptl_abinit.log
../pFOLD.${compiler} aa 2ptl _ -constant_seed -nstruct 1 -status -paths ../2ptl/paths.txt > 2ptl_abinit.log

cp aa2ptl0001.pdb ../2ptl

echo;echo idealize test...
echo ../pFOLD.${compiler} -idealize -s 2ptl -constant_seed -paths ../2ptl/paths.txt > 2ptl_idealize.log
../pFOLD.${compiler} -idealize -s 2ptl -constant_seed -paths ../2ptl/paths.txt > 2ptl_idealize.log
#
# Rename the outfiles.
#
cp aa2ptl0001.pdb ${dir}/aa2ptl000${PARAMETER}.pdb
cp 2ptl_idl.pdb ${dir}/2ptl_idl${PARAMETER}.pdb

# View the output obtained during ROSETTA's computation.
echo "+------------View the output files produced during the computation------------+ "
echo;echo "Content of the folder 2ptl ";ls -al ../2ptl
echo;echo "Content of the folder test ";ls -al `pwd`

echo; cd ${dir}
echo "###################################################################################################"
echo "RASTER-3D@Work!";echo
PROTEIN2RENDERING_1=aa2ptl000${PARAMETER}.pdb
PROTEIN2RENDERING_2=2ptl_idl${PARAMETER}.pdb
FILE2RENDERING_1=`echo ${PROTEIN2RENDERING_1} | awk -F'.' '{print $1}'`
FILE2RENDERING_2=`echo ${PROTEIN2RENDERING_2} | awk -F'.' '{print $1}'`
RASTERTOOLS=balls

echo "Start Rendering of : ${PROTEIN2RENDERING_1} and ${PROTEIN2RENDERING_2}.."
echo "..using the $RASTERTOOLS package." 

# Draw smooth $RASTERTOOLS with default color scheme 2, 
# save description (with header records) in ribbon.r3d. 
if [ "$RASTERTOOLS" == "balls" ] || [ "$RASTERTOOLS" == "rods" ]; then
	echo "Creation of ribbon.r3d file..(1st case)"
	cat "$PROTEIN2RENDERING_1" | "$RASTERTOOLS" > ribbon_1.r3d
	cat "$PROTEIN2RENDERING_2" | "$RASTERTOOLS" > ribbon_2.r3d
	
else 	
	echo "Creation of ribbon.r3d file..(2nd case)"
	cat "$PROTEIN2RENDERING_1" | rods > ribbon_1.r3d
	cat "$PROTEIN2RENDERING_2" | rods > ribbon_2.r3d
fi
 
if [ "$RASTERTOOLS" == "rings3d" ]; then
	rings3d -bases < "$PROTEIN2RENDERING_1" > temp.2
	# Combine the two descriptions and render as PNG image file.
	echo "Creation of temp.2 file.."
	# Combine the three descriptions and render as PNG image file
	echo "Creation of IMAGE.PNG!".
	cat ribbon_1.r3d temp.2 | render -png > image.png	
	# Remove the temponary file.
	rm temp.2
else
	echo "Creation of IMAGE.PNG!!"
	# Combine the two descriptions and render as PNG image file.
	cat ribbon_1.r3d | render -png  > ${FILE2RENDERING_1}.png
	cat ribbon_2.r3d | render -png  > ${FILE2RENDERING_2}.png
fi

ls -al
# Remove the temponary files.
#rm *.pdb
rm ribbon_1.r3d ribbon_2.r3d
#
#
#################################################
# Start the 3D-rendering using MolScript v2.1.2 #
# Copyright (C) 1997-1998 Per J. Kraulis        #
# ref: P.J. Kraulis, J. Appl. Cryst. (1991)     #
# vol 24, pp 946-950                            #
#       http://www.avatar.se/molscript/         #
#################################################
#
#
echo; cd ${dir}

# Create the wrl file.
echo "MolScript@Work!!!"
echo "molauto -nice -turns ${PROTEIN2RENDERING_1} | molscript -vrml > ${FILE2RENDERING_1}.wrl"
`molauto -nice -turns ${PROTEIN2RENDERING_1} | molscript -vrml > ${FILE2RENDERING_1}.wrl`
echo "molauto -nice -turns ${PROTEIN2RENDERING_2} | molscript -vrml > ${FILE2RENDERING_2}.wrl"
`molauto -nice -turns ${PROTEIN2RENDERING_2} | molscript -vrml > ${FILE2RENDERING_2}.wrl`
ls -al
echo "###################################################################################################"
# 
# Remove the .gz files.
cd ${LOCAL_DIR}
rm -f *.gz
echo
echo "###################################################################################################"
echo "#												  #"
echo "# @@@@ @@@@@ @@@@ @@@@ @@@@@ @@@@@ @@@@@  	@@@@@ @   @ @@@@@		@     @@@@@ @@@@@ #"
echo "# @  @ @   @ @    @      @     @   @   @	@   @ @@  @ @	@ 		@     @	    @	  #"
echo "# @@@@ @   @ @@@@ @@@    @     @   @@@@@	@@@@@ @ @ @ @   @		@     @@@   @	  #"
echo "# @ @  @   @    @ @      @     @   @   @	@   @ @  @@ @   @		@     @	    @	  #"
echo "# @  @ @@@@@ @@@@ @@@@   @     @   @   @	@   @ @   @ @@@@@		@@@@@ @	    @@@@@ #"
echo "#												  #"
echo "###################################################################################################"
#Check if it's feasible to upload the .pdb files on the LFC Catalog.
if [[ "x$GLITE_UI" = "xTRUE" || "x$GLITE_UI" != "xTRUE" ]] ; then
        lcg-lr -v --vo gilda  lfn:/grid/gilda/ROSETTA/aa2ptl000${PARAMETER}.pdb
        lcg-lr -v --vo gilda  lfn:/grid/gilda/ROSETTA/2ptl_idl${PARAMETER}.pdb
fi

if [ $? -eq 1 ]; then
	echo;echo "					======> Uploading files to the Catalog <======";echo
        #Store of this files onto the SE.
        if [[ "x$GLITE_UI" = "xTRUE" || "x$GLITE_UI" != "xTRUE" ]] ; then
                echo lcg-cr -v --vo gilda -l lfn:/grid/gilda/ROSETTA/aa2ptl000${PARAMETER}.pdb -d aliserv6.ct.infn.it file:`pwd`/aa2ptl000${PARAMETER}.pdb
                lcg-cr -v --vo gilda -l lfn:/grid/gilda/ROSETTA/aa2ptl000${PARAMETER}.pdb -d aliserv6.ct.infn.it file:`pwd`/aa2ptl000${PARAMETER}.pdb
                echo
                echo lcg-cr -v --vo gilda -l lfn:/grid/gilda/ROSETTA/2ptl_idl${PARAMETER}.pdb -d aliserv6.ct.infn.it file:`pwd`/2ptl_idl${PARAMETER}.pdb
                lcg-cr -v --vo gilda -l lfn:/grid/gilda/ROSETTA/2ptl_idl${PARAMETER}.pdb -d aliserv6.ct.infn.it file:`pwd`/2ptl_idl${PARAMETER}.pdb
        fi
        echo "#######################################################################################"
else
        echo;echo "======> Warning! The same run number exists on the Catalog. <======"
	echo "See below the details of these entries."
        lcg-lr --vo gilda lfn:/grid/gilda/ROSETTA/aa2ptl000${PARAMETER}.pdb
        lcg-lr --vo gilda lfn:/grid/gilda/ROSETTA/2ptl_idl${PARAMETER}.pdb
fi

echo; echo "That's all folks! Have a nice day ;-)"
echo "+--------------------------------------------------------+ FINISH ROSETTA at " `date`
#
#############################
#   Calculate the timing    #	
# for ROSETTA's execution.  #
#############################
#
START=`cat ${WORKING_DIR}/std${PARAMETER}.out | grep -i "START ROSETTA at" | awk -F'at' '{print $2}'`
FINISH=`cat ${WORKING_DIR}/std${PARAMETER}.out | grep -i "FINISH ROSETTA at" | awk -F'at' '{print $2}'`
echo -e '\e[0m[\e[32m START \e[0m ]' ${START}>${WORKING_DIR}/timing
echo -e '\e[0m[\e[32m FINISH \e[0m]' ${FINISH}>>${WORKING_DIR}/timing
