      module fruit_module
      implicit none

      private
      public :: write_header, write_data,                               &
     &          print_error,                                            &
     &          read_header, read_data

!...+&v..1....+....2....+....3....+....4....+....5....+....6....+....72v&
      character(*), parameter         :: sentinel              = 'fruit'
      integer,      parameter, public :: ok                         =  0
      integer,      parameter, public :: unable_to_write_to_unit    =  1
      integer,      parameter, public :: type_and_data_do_not_agree =  2
      integer,      parameter, public :: bad_size_of_argument       =  4
      integer,      parameter, public :: unable_to_read_from_unit   =  8
      integer,      parameter, public :: file_is_fruitless          = 16
!...+&v..1....+....2....+....3....+....4....+....5....+....6....+....72v&

      integer                       :: i, j, ios

      character(len=len(sentinel))  :: value_of_sentinel           = ' '
      integer, save                 :: value_of_max_length         = 0
      integer, save                 :: value_of_no_lines           = 0

      contains

      subroutine write_header (unit, max_length,  no_lines, status)

      integer, intent(in)  :: unit
      integer, intent(in)  :: max_length
      integer, intent(in)  :: no_lines
      integer, intent(out) :: status

      status  = ok

      write(unit, *, iostat=ios) sentinel//char(013)
      if (ios /= 0) then
        status = status + unable_to_write_to_unit
      else
        write(unit, *) max_length, no_lines
      endif
      value_of_no_lines    = no_lines
      value_of_max_length = max_length
      end subroutine

      subroutine read_header (unit, max_length,  no_lines, status)

      integer, intent(in)  :: unit
      integer, intent(out) :: max_length
      integer, intent(out) :: no_lines
      integer, intent(out) :: status

      status  = ok

      read(unit, *, iostat=ios) value_of_sentinel
      if (ios /= 0) then
        status = status + unable_to_read_from_unit
        return
      else
        if (value_of_sentinel /= sentinel) then
          status = status + file_is_fruitless
          return
        else
          read (unit, *) max_length, no_lines
        endif
      endif
      value_of_no_lines    = no_lines
      value_of_max_length = max_length
      end subroutine


      subroutine write_data (unit, type, data, status)

      integer, intent(in)                  :: unit
      integer, intent(in), dimension(:, :) :: type
      real,    intent(in), dimension(:, :) :: data
      integer, intent(out)                 :: status

      integer                              :: max_length
      integer                              :: no_lines

      status  = ok
      if (size(type) /= size(data)) then
        status = status + type_and_data_do_not_agree
      else
        max_length = size(type, dim=1)
        no_lines   = size(type, dim=2)
!...+&v..1....+....2....+....3....+....4....+....5....+....6....+....72v&
        if ((max_length /= value_of_max_length) .or.                    &
     &      (no_lines   /= value_of_no_lines)                           &
     &     ) then
          status = status + bad_size_of_argument
        else
          do j = no_lines, 1, -1
            do i = 1, max_length
              write(unit, *) type(i, j), data(i, j)
            end do
          end do
        end if
      end if
      end subroutine

      subroutine read_data (unit, type, data, status)

      integer, intent(in)                   :: unit
      integer, intent(out), dimension(:, :) :: type
      real,    intent(out), dimension(:, :) :: data
      integer, intent(out)                  :: status

      integer                              :: max_length
      integer                              :: no_lines

      status  = ok
      if (size(type) /= size(data)) then
        status = status + type_and_data_do_not_agree
      else
        max_length = size(type, dim=1)
        no_lines   = size(type, dim=2)
!...+&v..1....+....2....+....3....+....4....+....5....+....6....+....72v&
        if ((max_length /= value_of_max_length) .or.                    &
     &      (no_lines   /= value_of_no_lines)                           &
     &     ) then
          status = status + bad_size_of_argument
        else
          do j = no_lines, 1, -1
            do i = 1, max_length
              read(unit, *) type(i, j), data(i, j)
            end do
          end do
        end if
      end if
      end subroutine

      subroutine print_error(status)
      integer, intent(in)                 :: status
      if (status /= ok) then
        write(0, *) 'status = ', status
      end if
      end subroutine

      end module
