#!/bin/sh
#

# this parameter is the binary to be executed
EXE=$1
# this parameter is the number of CPU's to be reserved for parallel execution
CPU_NEEDED=$2

# prints the name of the master node
echo "Running on: $HOSTNAME"
echo "*************************************"

if [ -f "$PWD/.BrokerInfo" ] ; then
	TEST_LSF=`edg-brokerinfo getCE | cut -d/ -f2 | grep lsf`
else
	TEST_LSF=`ps -ef | grep sbatchd | grep -v grep`
fi

if [ "x$TEST_LSF" = "x" ] ; then
	# prints the name of the file containing the nodes allocated for parallel execution
	echo "PBS Nodefile: $PBS_NODEFILE"
	# print the names of the nodes allocated for parallel execution
	cat $PBS_NODEFILE
	echo "*************************************"
	HOST_NODEFILE=$PBS_NODEFILE
else
	# print the names of the nodes allocated for parallel execution
	echo "LSF Hosts: $LSB_HOSTS"
	# loops over the nodes allocated for parallel execution
	HOST_NODEFILE=`pwd`/lsf_nodefile.$$

	for host in ${LSB_HOSTS}
	do
		echo $host >> ${HOST_NODEFILE}
	done
fi

echo "*************************************"
# prints the working directory on the master node
echo "Current dir: $PWD"
echo "*************************************"

for i in `cat $HOST_NODEFILE` ; do
	echo "Mirroring via SSH to $i"
	# creates the working directories on all the nodes allocated for parallel execution
	ssh $i mkdir -p `pwd`
	# copies the needed files on all the nodes allocated for parallel execution
	/usr/bin/scp -rp ./* $i:`pwd`
	# checks that all files are present on all the nodes allocated for parallel execution
	echo `pwd`
	ssh $i ls `pwd`
	# sets the permissions of the files
	ssh $i chmod 755 `pwd`/$EXE
	ssh $i ls -alR `pwd`
	echo "@@@@@@@@@@@@@@@"
done

# execute the parallel job with mpirun
echo "*********************************"
echo "Executing $EXE"
chmod 755 $EXE
ls -l
mpirun -np $CPU_NEEDED -machinefile $HOST_NODEFILE `pwd`/$EXE > executable.out
echo "*********************************" 
