#!/bin/bash

#/**********************************************************************
# *                 III TTTTTTT U     U                                *
# *                  I     T    U     U                                *
# *                  I     T    U     U                                *
# *                  I     T    U     U                                *
# *                 III    T    UUUUUUU                                *
# *                                                                    *
# *  ITU Informatics Institute                                         *
# *  Computational Science & Engineering Dept.                         *
# *  Istanbul / TURKIYE                                                *
# *                                                                    *
# *  By: Ozden AKINCI                                                  *
# *  E-mail: akinci@be.itu.edu.tr                                      *
# *  Date: 10.11.2005                                                  *
# *                                                                    *
# *  Associated files :                                                *
# *       > JDL File  : foamparallel_?CPU.jdl                          *
# *       > Script    : openFOAMparallel.sh                            *
# *	  > InputFile : test??.tar                                     *
# *       > OutputFile: test??_out.tar                                 *
# *                                                                    *
#**********************************************************************/


if [ $# -lt 2 ]
then
	echo "usage: openFOAM.sh <FILE> <CPU>"
	exit 1
fi

# $1 argument must be with out extension, the argument will refer to
# the <FILE>.tar
FILE=$1

# $2 argument is the number of CPUs needed by the job.
CPU=$2

# HOSTPORT will be used to get GUI, if it is needed.
HOSTPORT=9988

# If TEST value is defined, the script will need to a machinefile,
# which is defined in LAMFILE, and also it is necessary that defined in 
# the InputSandbox attribute of jdl file.
TEST=

# MPIRUN is the name of the mpirun command for LAM/MPI of OpenFOAM.
# !!! use mpirunFoam instead of mpirun !!! #
MPIRUN=mpirunFoam

# LAMRSH is the communication way of the LAM/MPI
export LAMRSH=ssh


# ROOT is the name of the root directory in the tar file.
# File hierarchy is as $FILE/$ROOT
ROOT=icoFoam

# CASE is the name of the case in the ROOT directory. 
# CASE is also a directory.
# The file hierarchy is as $FILE/$ROOT/$CASE
CASE=cavity

# TARFILE is name of the $1 argument with .tar extension.
# Name of TARFILE must be same with the filename 
# in the InputSandbox attribute of jdl file.
TARFILE=$FILE.tar

# OUTPUTTAR is the output filename of the program. It takes FILE is base, 
# and adds _out.tar extension
OUTPUTTAR=${FILE}_out.tar
WD=`pwd`
LAMFILE=$WD/machinefile
echo "TARFILE: $TARFILE"
echo "HOSTPORT: $HOSTPORT"
echo "Number of CPU=$CPU"
#OpenFOAM settings 
# The line below is necessary to run an OpenFOAM job.
#################################################
. /opt/OpenFOAM/OpenFOAM-1.2/.OpenFOAM-1.2/bashrc

export RUN=`pwd`
export FOAM_RUN=$RUN
export HOSTPORT=$HOSTPORT

# ENVIRONMENTS will be sent to all of the hosts
ENVIRONMENTS=WM_LINK_LANGUAGE,VTK_DIR,MANPATH,FOAM_SOLVERS,FOAM_APPBIN,FOAM_TUTORIALS,FOAM_JOB_DIR,WM_ARCH,WM_JAVAC_OPTION,FOAM_LIB,FOAMX_CONFIG,ParaView_INST_DIR,FOAM_MPI_LIBBIN,FOAMX_PATH,FOAM_APP,WM_PROJECT_VERSION,CMAKE_HOME,MICO_VERSION,LD_LIBRARY_PATH,MPI_BUFFER_SIZE,WM_COMPILER_LIB_ARCH,FOAM_SRC,ParaView_DIR,WM_PROJECT_INST_DIR,FOAM_UTILITIES,WM_PROJECT_DIR,WM_COMPILER_ARCH,JAVA_PATH,WM_DECOMP_LIBS,WM_PROJECT,WM_COMPILER,WM_MPLIB,PATH,FOAM_USER_LIBBIN,CEI_ARCH,WM_COMPILER_DIR,JAVA_HOME,LAMHOME,PV_INTERFACE_PATH,WM_DECOMP_INC,LAM_VERSION,FOAM_LIBBIN,WM_COMPILE_OPTION,WM_SHELL,ParaView_VERSION,WM_DIR,MICO_ARCH_PATH,FOAM_DOT_DIR,WM_PROJECT_USER_DIR,CEI_HOME,LAM_ARCH_PATH,CLASSPATH,FOAM_RUN,MICO_PATH,WM_OPTIONS,WM_PROJECT_LANGUAGE,FOAM_USER_APPBIN,JAVA_ROOT,SOURCE

if [ ! -d $FOAM_RUN ]
then
	mkdir -p $FOAM_RUN
fi

# prints the name of the master node
echo "Running on: $HOSTNAME"
echo "*************************************"

#################################################
# parallel
# determination of hosts
if [ "x$TEST" == "x" ]
then
	if [ -f "$PWD/.BrokerInfo" ] ; then
		echo "*************************************"
		echo "BrokerInfo exists."
		TEST_LSF=`edg-brokerinfo getCE | cut -d/ -f2 | grep lsf`
	else
		TEST_LSF=`ps -ef | grep sbatchd | grep -v grep`
		echo "BrokerInfo does NOT  exist."
	fi
		echo "TEST_LSF INFO: $TEST_LSF"

	if [ "x$TEST_LSF" = "x" ] ; then
		# prints the name of the file containing the nodes allocated for parallel execution
		echo "PBS Nodefile: $PBS_NODEFILE"
		# print the names of the nodes allocated for parallel execution
		cat $PBS_NODEFILE
		echo "*************************************"
		HOST_NODEFILE=$PBS_NODEFILE
	else
		# print the names of the nodes allocated for parallel execution
		echo "LSF Hosts: $LSB_HOSTS"
		# loops over the nodes allocated for parallel execution
		HOST_NODEFILE=`pwd`/lsf_nodefile.$$

		for host in ${LSB_HOSTS}
		do
			echo $host >> ${HOST_NODEFILE}
		done
	fi
	
	LAMFILE=${HOST_NODEFILE}
else
	HOST_NODEFILE=$LAMFILE
fi

echo " The ${HOST_NODEFILE}"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
cat ${HOST_NODEFILE}
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
echo "List of the files on $HOSTNAME"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
pwd
ls -la
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
echo "Untar procedure for $FILE"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
tar xvf $TARFILE
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
echo "List of files after untar process"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
ls -la
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
echo "List of Files on FOAM_RUN directory"
echo "FOAM_RUN=$FOAM_RUN"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
ls -la $FOAM_RUN/
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"

#########################################################
# OPENFOAM APPLICATOINS (sequential section)
#########################################################
cd $FILE
#blockMesh $FOAM_RUN/$FILE/$ROOT $CASE >blockMesh.out$$ 2>&1
blockMesh $FOAM_RUN/$FILE/$ROOT $CASE 2>&1
RESULT=$?
if [ "$RESULT" == "0" ]
then
	echo "BlochMesh section ended succesfully"
else
	echo "There is an error occured while blockMesh was working"
	exit 98
fi
#decomposePar $FOAM_RUN/$FILE/$ROOT $CASE >decomposePar.out$$ 2>&1
decomposePar $FOAM_RUN/$FILE/$ROOT $CASE 2>&1
RESULT=$?
if [ "$RESULT" == "0" ]
then
	echo "decomposePar section ended succesfully"
else
	echo "There is an error occured while decomposePar was working"
	exit 97
fi

# END of OPENFOAM APPLICATOINS (sequential section)

cd ../
# mirroring via SSH
for i in `cat $HOST_NODEFILE| grep -v $HOSTNAME` ; do
        echo "Mirroring via SSH to $i"
        # creates the working directories on all the nodes allocated for parallel execution
        ssh $i mkdir -p `pwd`
        # copies the needed files on all the nodes allocated for parallel execution
        /usr/bin/scp -rp ./* $i:`pwd`
        echo "list of files on host $i"
        echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
        /usr/bin/ssh $i ls -la `pwd`
        echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
done
# end of mirroring via SSH

echo " list of files on `pwd` directory"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
ls -la 
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
echo "@@@@@@@@@@@@@@@"
# end of parallel
#################################################
#########################################################
# OPENFOAM APPLICATOINS for parallel
#########################################################
#cd $FILE
echo "lam host file: $LAMFILE"
echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
cat $LAMFILE
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
#lamboot -v -d $LAMFILE >lamboot.out$$
lamboot -v -d $LAMFILE 2>&1
RESULT=$?
if [ "$RESULT" == "0" ]
then
	echo "lamboot section ended succesfully"
else
	echo "There is an error occured while lamboot was working"
	exit 96
fi
lamnodes 
# !!! use mpirunFoam instead of mpirun !!! #
#$MPIRUN -np $CPU -x $ENVIRONMENTS $ROOT $FOAM_RUN/$FILE/$ROOT $CASE -parallel >$MPIRUN.out$$ 2>&1
$MPIRUN -np $CPU -x $ENVIRONMENTS $ROOT $FOAM_RUN/$FILE/$ROOT $CASE -parallel 2>&1
RESULT=$?
if [ "$RESULT" == "0" ]
then
	echo "$MPIRUN section ended succesfully"
else
	echo "There is an error occured while $MPIRUN was working"
	exit 95
fi

for i in `cat $HOST_NODEFILE|grep -v $HOSTNAME` ; do
        echo "Mirroring via SSH from $i to $HOSTNAME"
        /usr/bin/scp -rp $i:$FOAM_RUN/$FILE/$ROOT/$CASE/processor* $FOAM_RUN/$FILE/$ROOT/$CASE/
done
# halting of lam
lamhalt -d
# END of OPENFOAM APPLICATOINS for parallel

#reconstructPar  $FOAM_RUN/$FILE/$ROOT $CASE > reconstructPar.out$$ 2>&1
reconstructPar  $FOAM_RUN/$FILE/$ROOT $CASE 2>&1
RESULT=$?
if [ "$RESULT" == "0" ]
then
	echo "reconstructPar section ended succesfully"
else
	echo "There is an error occured while reconstructPar was working"
	exit 94
fi

echo "Calculation for $ROOT with $CASE has been done"
#cd ../
tar cvf $OUTPUTTAR $FILE
echo "The output $FILE is archived as $OUTPUTTAR"
echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
echo "...The End..."
