      module shade_module

      implicit none

      integer, save     :: int_value_min  = 0
      integer, save     :: int_value_max  = 0
      real,    save     :: real_value_min = 0
      real,    save     :: real_value_max = 0

      character(len=1), allocatable, save, dimension(:) :: saved_colours
      integer, save     :: min_colour  = 0
      integer, save     :: max_colour  = 0

      interface set_min_and_max
        module procedure set_min_and_max_i, set_min_and_max_r
      end interface

      interface colour
        module procedure colour_i, colour_r
      end interface

      contains

      subroutine set_colours (colours)
      character(len=*), intent(in), dimension (:) :: colours

      max_colour = size(colours)
      min_colour = 1

      call del_colours()
      allocate(saved_colours(size(colours)))
      saved_colours = colours

      end subroutine

      subroutine del_colours ()

      if (allocated(saved_colours)) then
        deallocate(saved_colours)
      end if

      end subroutine

      subroutine set_min_and_max_i(i, j)
      integer, intent(in) :: i
      integer, intent(in) :: j
      int_value_min = i
      int_value_max = j
      end subroutine

      subroutine set_min_and_max_r(x, y)
      real, intent(in) :: x
      real, intent(in) :: y
      real_value_min = x
      real_value_max = y
      end subroutine

      character function colour_r (r)
      real, intent(in) :: r
!...+....1....+....2....+....3....+....4....+....5....+....6....+....7.v&
      real :: rmin
      real :: rmax
      integer :: cmin
      integer :: cmax
      rmin = real_value_min
      rmax = real_value_max
      cmin = min_colour
      cmax = max_colour
      colour_r = saved_colours(max(min_colour,min(max_colour,           &
     &              int((r-rmin)*(cmax-cmin)*(cmax-cmin+1)/(cmax-cmin)/ &
     &                             (rmax-rmin)+cmin)                    &
     &                        )))
      end function

      character function colour_i (i)
      integer, intent(in) :: i
!...+....1....+....2....+....3....+....4....+....5....+....6....+....7.v&
      integer :: imin
      integer :: imax
      integer :: cmin
      integer :: cmax
      imin = int_value_min
      imax = int_value_max
      cmin = min_colour
      cmax = max_colour
      colour_i = saved_colours(max(min_colour,min(max_colour,           &
     &                  (i-imin)*(cmax-cmin)*(cmax-cmin+1)/(cmax-cmin)/ &
     &                              (imax-imin)+cmin                    &
     &                        )))
      end function

      end module
